
/*---------------------------------------------------------------------------

    access.c -- determine access permissions of a file

    Written By: S.E. Norris

    Usage:
       character filename   ! name of file to check permissions of
       character mode       ! permissions to check for
       integer   access     ! function name
       integer   ierror
       external  access
       integer addr ! returned address of memory block

       ierror = access(filename, mode)

    mode contains permissions to check for: 'r', 'w', 'x'. To check for
    existance use a blank character ' '.

    $RCSfile: access.c,v $
    $Author: norris $
    $Revision: 1.6 $
    $Date: 1996/11/28 12:00:26 $
 
    $Log: access.c,v $
    Revision 1.6  1996/11/28 12:00:26  norris
    General clean up of the code.

    Revision 1.5  1996/04/01 06:37:39  norris
    Added sys/errno.h to the include list, for the HP's

    Revision 1.4  1996/03/31 05:11:28  norris
    Fixed another bug, to prevent zero length strings being passed in for testing.

    Revision 1.3  1996/03/29  07:56:02  norris
    Fixed a bug, in that the routine didn't trap bad arguments being passwd in.

    Revision 1.2  1996/03/29 05:37:00  norris
    Removed macro type object names. Cleaned up header files.

    Revision 1.1  1996/03/27  08:53:44  norris
    Initial revision

 ---------------------------------------------------------------------------*/

#include <sys/errno.h>
#include <sys/param.h>
#include <errno.h>
#include "libU77.h"

integer access_(char *name, char *mode, ftnlen namlen, ftnlen modlen)
{
  char buf[MAXPATHLEN];
  int m = 0;

  if (namlen >= sizeof buf) {
    return((integer) (errno=F_ERARG));
  }

  if (modlen == 0) {
    return((integer) (errno=F_ERARG));
  }

  fcopyin(name, namlen, buf);
  if (buf[0] == '\0') {
    return((integer) (errno=ENOENT));
  }

  if (access(buf, 0) < 0) {
    return((integer)errno);
  }
	
  while (modlen--) {
    switch(*mode++) { 
    case 'x':
      m |= 1;
      break;
      
    case 'w':
      m |= 2;
      break;
	  
    case 'r':
      m |= 4;
      break;

    case ' ':
      break;

    default:
      return((integer) (errno=EINVAL));
      break;
    }
  }
 
  if (m > 0 && access(buf, m) < 0) {
    return((integer) errno);
  }

  return((integer) 0);
}
