
/*---------------------------------------------------------------------------
    etime() -- returns the elapsed cpu time used by the process

    Written By: S.E. Norris

    Usage:

      real total
      real tarray(2)
      real etime
      external etime

      total = etime(tarray)

    $RCSfile: etime.c,v $
    $Author: norris $
    $Revision: 1.8 $
    $Date: 1996/04/01 06:49:58 $
 
    $Log: etime.c,v $
    Revision 1.8  1996/04/01 06:49:58  norris
    Added the sys/time.h for the HP's.

    Revision 1.7  1996/04/01 06:38:10  norris
    Changed CLK_TIC to CLOCKS_PER_SEC, which is correct ANSI.

    Revision 1.6  1996/03/31 23:55:32  norris
    Cleaned up so _TIMES macro will switch between getrusage() and times().

    Revision 1.5  1996/03/29  05:37:00  norris
    Removed macro type object names. Cleaned up header files.

    Revision 1.4  1996/03/28  10:44:53  norris
    Changed to use time.h not sys/time.h

    Revision 1.3  1996/03/28  09:18:59  norris
    Ported to POSIX/SunOS.

    Revision 1.2  1996/03/28 00:31:30  norris
    Added a trailing underscore to the function name.

    Revision 1.1  1996/03/26  00:50:02  norris
    Initial revision

  ---------------------------------------------------------------------------*/

#include <sys/types.h>
#ifdef _TIMES
#include <sys/time.h>
#include <sys/times.h>
#include <limits.h>
#else
#include <time.h>
#endif
#include <sys/resource.h>
#include "libU77.h"

real etime_(real t[2])
{
#ifdef _TIMES
  struct tms tm;

  times(&tm);
  t[0] = (real) (tm.tms_utime + tm.tms_cutime) / (real) CLOCKS_PER_SEC;
  t[1] = (real) (tm.tms_stime + tm.tms_cstime) / (real) CLOCKS_PER_SEC;
#else
  struct rusage tm;
  float meg = 1000000.0;

  getrusage(RUSAGE_SELF, &tm);
  t[0] = (real) (tm.ru_utime.tv_sec + (tm.ru_utime.tv_usec) / meg);
  t[1] = (real) (tm.ru_stime.tv_sec + (tm.ru_stime.tv_usec) / meg);
#endif

  return (real) (t[0]+t[1]);
}
