
/*---------------------------------------------------------------------------

    system.c -- execute a UNIX command.

    Written By: S.E. Norris

    Usage:

      integer   exit
      integer   system ! external function.
      character command
      external  system 

      exit = system(command) ! executes the command

    $RCSfile: system.c,v $
    $Author: norris $
    $Revision: 1.4 $
    $Date: 1996/11/28 12:00:26 $
 
    $Log: system.c,v $
    Revision 1.4  1996/11/28 12:00:26  norris
    General clean up of the code.

    Revision 1.3  1996/03/31 05:13:18  norris
    Removed use of the NCARGS macro.

    Revision 1.2  1996/03/29  05:37:00  norris
    Removed macro type object names. Cleaned up header files.

    Revision 1.1  1996/03/29  00:48:08  norris
    Initial revision

  ---------------------------------------------------------------------------*/

extern int errno;

#include <sys/param.h>
#include "libU77.h"

integer system_(char *s, ftnlen n)
{
  char buf[256];
  long i;

  if (n >= sizeof buf) {
    return(-(integer)(errno=F_ERARG));
  }

  fcopyin(s, n, buf);
  return(f_system(buf));
}


#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

integer f_system(char *s)
{
  int (*istat)(), (*qstat)();
  int status, pid, w;
  char *shname, *shell;

  if ((shell = getenv("SHELL")) == NULL) {
    shell = "/bin/sh";
  }

  if ((pid = fork()) == 0) {
    execl(shell, shell, "-c", s, 0);
    _exit(0);
  }

  istat = signal(SIGINT,  SIG_IGN);
  qstat = signal(SIGQUIT, SIG_IGN);
  while ((w = wait(&status)) != pid && w != -1) {
    ;
  }

  if (w == -1) {
    status = -1;
  }
  signal(SIGINT,  istat);
  signal(SIGQUIT, qstat);
  return((integer) status);
}
